/*
 * Copyright (C) 1997-2004 Andre Noll <noll@mathematik.tu-darmstadt.de>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "config.h"

#include <sys/stat.h>
#include <sys/timeb.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <signal.h>
#include <sys/signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <errno.h>
#include <limits.h>
#include <stdarg.h>
#include <stropts.h>
#include <poll.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <libgen.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <ctype.h>

/* some internal constants */
#define STRINGSIZE 4096
#define MAXLINE 255


#define MIN(a,b) (a) < (b) ? (a) : (b) 
#define MAX(a,b) (a) > (b) ? (a) : (b) 
#define ABS(a) (a) > 0 ? (a) : -(a)

/* Loglevels */
#define DEBUG 1
#define VERBOSE 2
#define INFO  3
#define NOTICE 4
#define WARNING 5
#define ERROR 6
#define CRIT 7
#define EMERG 8

/* used in several contexts */
#define SET 0
#define GET 1
#define INCR 2
#define RESET 3


/* These are the permission flags that can be set individually 
 * for any command 
 */
#define DB_READ 1
#define DB_WRITE 2
#define AFS_READ 4
#define AFS_WRITE 8

#define MODUS_READ 1
#define MODUS_WRITE 2

#define COPYRIGHT "Copyright (c) 1997-2004 by Andre Noll"

#define LICENSE "This is free software with ABSOLUTELY NO WARRANTY. " \
	"See COPYING for details."

#define EMAIL "Send bug reports and " \
	"questions to <noll@mathematik.tu-darmstadt.de>."

#define AWAITING_DATA_MSG "\nAwaiting Data."
#define PROCEED_MSG "\nProceed."
#define EOC_MSG "\nEnd of Command."

struct gui_command {
	char *key;
	char *name;
	char *description;
	void (*handler)(int);
};

/* net */
struct hostent *get_host_info(char *);
int get_socket(void);
void init_sockaddr(struct sockaddr_in*, int, struct hostent*);
int do_connect(int, struct sockaddr_in *);
int send_buffer(int, char *);
int send_buffer_ll(int, char *, int);
int send_bin_buffer(int, char *, ssize_t);
int send_va_buffer(int, char *, ...);
int recv_buffer(int, char *, ssize_t);
int recv_bin_buffer(int, char *, ssize_t);
int setserversockopts(int);
int do_bind(int, struct sockaddr_in*);
int do_accept(int, struct sockaddr_in *);

/* gui */
int check_buf_for_items(char *, int, void (*)(char *), int);
int file_exists(const char *);
FILE *open_stat_pipe(void);
FILE *popen_read_client_cmdline(char *);
FILE *popen_read_client_cmdline_pid(pid_t *, char *);
FILE *popen_read_client_pid(pid_t *, char **);

/* exec */
FILE *popen_read_vp(const char*, char * const[]);
FILE *popen_write_vp(const char*, char * const[]);
FILE *popen_read_client(char **);
FILE *popen_read_dbtool(char **);
int split_args(char *, char ***, int);
FILE *popen_read_cmdline_pid(pid_t *, char *);
FILE *popen_write_cmdline_pid(pid_t *, char *);
int exec_cmdline_pid_bg(pid_t *, const char *);

/* strings */
void chop(char*);
void *para_realloc(void*, size_t);
void *para_malloc(size_t);
char *para_strdup(const char *);
char *para_basename(char *);
char *make_message(const char *, ...);
char *para_strcat(char *, const char *);
struct regex_list {
	char *regex;
	char *replacement;
};

/* signal */
int para_signal_init(int *spipe, void (*the_log_fn)(int , char* ,...));
int para_install_sighandler(int);
void para_reap_children(void);
pid_t para_reap_child(void);
int para_next_signal(void);

char *s_a_r_list(struct regex_list *, char *);
